#
# Example of a GDELT 2.0 record.
#
example_record = {
    "_id": 1034618754,
    "Day": 20210318,
    "MonthYear": 202103,
    "Year": 2021,
    "FractionDate": 2021.2137,
    "Actor1Code": "COP",
    "Actor1Name": "POLICE",
    "Actor1CountryCode": "",
    "Actor1KnownGroupCode": "",
    "Actor1EthnicCode": "",
    "Actor1Religion1Code": "",
    "Actor1Religion2Code": "",
    "Actor1Type1Code": "COP",
    "Actor1Type2Code": "",
    "Actor1Type3Code": "",
    "Actor2Code": "JUD",
    "Actor2Name": "PROSECUTOR",
    "Actor2CountryCode": "",
    "Actor2KnownGroupCode": "",
    "Actor2EthnicCode": "",
    "Actor2Religion1Code": "",
    "Actor2Religion2Code": "",
    "Actor2Type1Code": "JUD",
    "Actor2Type2Code": "",
    "Actor2Type3Code": "",
    "IsRootEvent": 0,
    "EventCode": "010",
    "EventBaseCode": "010",
    "EventRootCode": "01",
    "QuadClass": 1,
    "GoldsteinScale": 0,
    "NumMentions": 54,
    "NumSources": 9,
    "NumArticles": 54,
    "AvgTone": -10.4477611940299,
    "Actor1Geo_Type": 4,
    "Actor1Geo_Fullname": "London, London, City of, United Kingdom",
    "Actor1Geo_CountryCode": "UK",
    "Actor1Geo_ADM1Code": "UKH9",
    "Actor1Geo_ADM2Code": "40110",
    "Actor1Geo_Lat": 51.5,
    "Actor1Geo_Long": -0.116667,
    "Actor1Geo_FeatureID": "-2601889",
    "Actor2Geo_Type": "4",
    "Actor2Geo_Fullname": "London, London, City of, United Kingdom",
    "Actor2Geo_CountryCode": "UK",
    "Actor2Geo_ADM1Code": "UKH9",
    "Actor2Geo_ADM2Code": "40110",
    "Actor2Geo_Lat": 51.5,
    "Actor2Geo_Long": -0.116667,
    "Actor2Geo_FeatureID": "-2601889",
    "ActionGeo_Type": 4,
    "ActionGeo_Fullname": "London, London, City of, United Kingdom",
    "ActionGeo_CountryCode": "UK",
    "ActionGeo_ADM1Code": "UKH9",
    "ActionGeo_ADM2Code": "40110",
    "ActionGeo_Lat": 51.5,
    "ActionGeo_Long": -0.116667,
    "ActionGeo_FeatureID": "-2601889", "DATEADDED": "20220318143000",
     "SOURCEURL": "https://www.lancashiretelegraph.co.uk/news/20003985.depraved-sex-predator-jailed-42-years-murder-violent-spree/",
}




from setuptools import setup, find_packages
import os
import glob


this_directory = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

VERSIONFILE="gdelttools/_version.py"
with open(VERSIONFILE, "rt") as vfile:
    for line in vfile:
        line = line.strip()
        (lhs, equals, rhs) = line.partition( "=")
        if lhs.strip() == "__version__":
            rhs = rhs.strip()
            version_string = rhs.strip('"')
pyfiles = [f for f in os.listdir(".") if f.endswith(".py")]


setup(
    name="gdelttools",
    version=version_string,

    author="Joe Drumgoole",
    author_email="joe@joedrumgoole.com",
    description="A set of tools to support downloading GDELT data",
    long_description=long_description,
    long_description_content_type='text/markdown',
    license="Apache 2.0",
    keywords="MongoDB GDELT dataset",
    url="https://github.com/jdrumgoole/gdelttools",

    install_requires=['pymongo',
                      'requests',
                    ],
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],

    # setup_requires=["pymongo",
    #                   "nose",
    #                   "dnspython",
    #                   "dateutils",
    #                   "configargparse",
    #                   "toml"],

    packages=find_packages(),

    data_files=[("test", glob.glob("data/*.ff") +
                 glob.glob("data/*.csv") +
                 glob.glob("data/*.txt"))],
    python_requires='>3.7',
    scripts=[],
    entry_points={
        'console_scripts': [
            'gdeltloader=gdelttools.gdeltloader:main',
        ]
    },

    test_suite='nose.collector',
    tests_require=['nose'],
)



import hashlib
import urllib
import zipfile
import requests
from io import BytesIO
from zipfile import ZipFile


def download_and_unzip(u: str):
    url_file = urllib.request.urlopen(u)

    with zipfile.ZipFile(BytesIO(url_file.read())) as my_zip_file:
        my_zip_file.extractall()


def compute_md5(file):
    hasher = hashlib.md5()
    with open(file, 'rb') as input:
        buf = input.read()
        hasher.update(buf)

    return hasher.hexdigest()


def local_path(url):
    filename_with_args = url.split('/')[-1]
    filename = filename_with_args.split('?')[0]
    return filename

def download_file(url):
    local_filename = local_path(url)
    # NOTE the stream=True parameter below
    with requests.get(url, stream=True) as r:
        r.raise_for_status()
        with open(local_filename, 'wb') as f:
            for i, chunk in enumerate(r.iter_content(chunk_size=8192), 1):
                if chunk:  # filter out keep-alive new chunks
                    print(".", end="")
                    if i % 80 == 0:
                        print("")
                    f.write(chunk)
                    # f.flush()
            if i % 80 != 0:
                print("")
    return local_filename


def extract_zip_file(filepath):
    zfile = zipfile.ZipFile(filepath)
    files = []
    for finfo in zfile.namelist():
        with zipfile.ZipFile(filepath) as archive:
            for zfilename in archive.namelist():
                text = archive.read(zfilename).decode(encoding="utf-8")
                with open(zfilename, "w") as output_file:
                    output_file.write(text)
            files.append(zfilename)

    return files



"""
Importer for GDELT 2.0 raw data set.
https://blog.gdeltproject.org/gdelt-2-0-our-global-world-in-realtime/

The master file list is here:
http://data.gdeltproject.org/gdeltv2/masterfilelist.txt

"""
import argparse
from typing import List

import requests
import sys
from datetime import datetime
import os
import pymongo

from gdelttools.web import local_path, download_file, compute_md5, extract_zip_file
from gdelttools.gdeltcsv import Downloader
from gdelttools._version import __version__

def download_zips(collection, file_list: List[str], overwrite=False):
    for f in file_list:
        with open(f, "r") as input_file:
            for line in input_file:
                size, md5, zipurl = line.split()
                size = int(size)
                md5 = str(md5)
                # print(f"{size}:{sha}:{zip}")
                local_zip_file = local_path(zipurl)
                if os.path.exists(local_zip_file) and not overwrite:
                    print(f"File '{local_zip_file}'exists locally cannot overwrite")
                    sys.exit(1)
                else:
                    print(f"Downloading:'{zipurl} size: {size}'")
                    download_file(zipurl)
                    computed_md5 = compute_md5(local_zip_file)

                if computed_md5 == md5:
                    print(f"Unzipping: '{local_zip_file}'")
                    local_csv_files = extract_zip_file(local_zip_file)
                    if collection:
                        collection.insert_one({"_id": zipurl,
                                               "ts": datetime.utcnow(),
                                               "local_zip_file": local_zip_file,
                                               "local_csv_file": local_csv_files[0],
                                               "size": size,
                                               "md5_zip_file": md5})
                else:
                    print(f"'{md5}' checksum for {zip} doesn't match computed\n"
                          f" checksum: {computed_md5} for {local_zip_file}")
                    sys.exit(0)


def main():

    parser = argparse.ArgumentParser(epilog=f"Version: {__version__}")

    parser.add_argument("--host",
                        help="MongoDB URI")

    parser.add_argument("--master",
                        default=False,
                        action="store_true",
                        help="GDELT master file [%(default)s]")

    parser.add_argument("--update",
                        default=False,
                        action="store_true",
                        help="GDELT update file [%(default)s]")

    parser.add_argument("--database", default="GDELT",
                        help="Default database for loading [%(default)s]")

    parser.add_argument("--collection", default="events_csv",
                        help="Default collection for loading [%(default)s]")

    parser.add_argument("--local", type=str,
                        help="load data from local list of zips")

    parser.add_argument("--overwrite",
                        default=False, action="store_true",
                        help="Overwrite files when they exist already")

    parser.add_argument("--download", default=False, action="store_true",
                        help="download zip files from master or local file")

    parser.add_argument("--metadata", action="store_true", default=False,
                        help="grab meta data files")
    args = parser.parse_args()

    # if args.ziplist == "master":
    #     url = args.master
    # else:
    #     url = args.incremental

    gdelt_md5_list = "http://data.gdeltproject.org/events/md5sums"
    gdelt_file_sizes = "http://data.gdeltproject.org/events/filesizes"

    files_collection = None
    if args.host:
        client = pymongo.MongoClient(host=args.host)
        db = client[args.database]
        files_collection = db["files"]
        events_collection = db[args.collection]

    if args.metadata:
        Downloader.get_metadata()

    input_file_list = []

    if args.master:
        print(f"Getting master file from '{Downloader.master_url}'")
        filename = Downloader.get_master_list()
        print(f"created: {filename}")
        input_file_list.append(filename)

    if args.update:
        print(f"Getting update file from '{Downloader.update_url}'")
        filename = Downloader.get_update_list()
        input_file_list.append(filename)
        print(f"created: {filename}")

    if args.local:
        if os.path.isfile(args.local):
            input_file_list.append(args.local)
        else:
            print(f"'{args.local}' does not exist")
            sys.exit(1)

    if args.download:
        if len(input_file_list) > 0:
            download_zips(files_collection, input_file_list, args.overwrite)
        else:
            print(f"No files listed for download")


if __name__ == "__main__":
    main()
